package kr.co.vividnext.sodalive.di

import android.content.Context
import com.google.gson.GsonBuilder
import kr.co.vividnext.sodalive.BuildConfig
import kr.co.vividnext.sodalive.common.ApiBuilder
import kr.co.vividnext.sodalive.main.MainViewModel
import kr.co.vividnext.sodalive.network.TokenAuthenticator
import kr.co.vividnext.sodalive.settings.terms.TermsApi
import kr.co.vividnext.sodalive.settings.terms.TermsRepository
import kr.co.vividnext.sodalive.settings.terms.TermsViewModel
import kr.co.vividnext.sodalive.user.UserApi
import kr.co.vividnext.sodalive.user.UserRepository
import kr.co.vividnext.sodalive.user.find_password.FindPasswordViewModel
import kr.co.vividnext.sodalive.user.login.LoginViewModel
import kr.co.vividnext.sodalive.user.signup.SignUpViewModel
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import org.koin.android.ext.koin.androidContext
import org.koin.androidx.viewmodel.dsl.viewModel
import org.koin.core.context.startKoin
import org.koin.dsl.module
import retrofit2.Retrofit
import retrofit2.adapter.rxjava3.RxJava3CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory

class AppDI(private val context: Context, isDebugMode: Boolean) {
    private val baseUrl = BuildConfig.BASE_URL

    private val otherModule = module {
        single { GsonBuilder().create() }
    }

    private val networkModule = module {
        single {
            val logging = HttpLoggingInterceptor()

            if (isDebugMode) {
                logging.setLevel(HttpLoggingInterceptor.Level.BODY)
            } else {
                logging.setLevel(HttpLoggingInterceptor.Level.NONE)
            }

            OkHttpClient().newBuilder()
                .addInterceptor(logging)
                .authenticator(TokenAuthenticator(get()))
                .build()
        }

        single {
            Retrofit.Builder()
                .baseUrl(baseUrl)
                .addConverterFactory(GsonConverterFactory.create())
                .addCallAdapterFactory(RxJava3CallAdapterFactory.create())
                .client(get())
                .build()
        }

        single { ApiBuilder().build(get(), UserApi::class.java) }
        single { ApiBuilder().build(get(), TermsApi::class.java) }
    }

    private val viewModelModule = module {
        viewModel { LoginViewModel(get()) }
        viewModel { SignUpViewModel(get()) }
        viewModel { TermsViewModel(get()) }
        viewModel { FindPasswordViewModel(get()) }
        viewModel { MainViewModel(get()) }
    }

    private val repositoryModule = module {
        factory { UserRepository(get()) }
        factory { TermsRepository(get()) }
    }

    private val moduleList = listOf(
        networkModule,
        viewModelModule,
        repositoryModule,
        otherModule
    )

    init {
        startKoin {
            androidContext(context)
            modules(moduleList)
        }
    }
}
